# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import Message
from driverng.fields import (
    String,
    Date
)
from driverng.splitted import (
    Splitted,
    Multiple,
    String as StringSplitted
)


class GentianeBase(Splitted):
    SEPARATOR = ";"


class ResultContainer(object):
    def __init__(self, code, value):
        self.code, self.value = code, value
        super(ResultContainer, self).__init__()

    def to_string(self):
        return ":".join([self.code, self.value])


class ResultList(list):
    def add(self, code, value):
        obj = ResultContainer(code, value)
        self.append(obj)
        return obj


class GentianeResult(object):
    FIELDS = (
        String("header"),
        String("identifiant", 30),
        String("nom_nomjf", 80),
        String("prenom", 20),
        Date("_naissance_j", fmt="%d"),
        Date("_naissance_m", fmt="%m"),
        Date("_naissance_a", fmt="%Y"),
        String("sexe"),  # M/F
        Date("_prelevement_time", fmt="%H:%M"),
        Date("_prelevement_date", fmt="%d/%m/%Y"),
        String("numero_dossier", 20),
        String("addresse1", 40),
        String("addresse2", 40),
        String("code_postal", 9),
        String("ville", 40),
        String("medecin", 20),
        String("commentaire", 80),
        String("nb_param"),
    )
    SEP = ";"

    def __init__(self, **kwargs):
        for field in self.FIELDS:
            setattr(self, field.name, kwargs.get(field.name, None))
        self.results = ResultList()
        self.date_prelevement = kwargs.get("date_prelevement", None)
        self.date_naissance = kwargs.get("date_naissance", None)
        super(GentianeResult, self).__init__()

    def parse(self, data):
        for i, value in enumerate(data.split(self.SEP)):
            try:
                field = self.FIELDS[i]
            except IndexError:
                self.results.add(*value.split(":"))
            else:
                setattr(self, field.name, field.to_python(value))
        if all([getattr(self, "_naissance_%s" % k) for k in ("j", "m", "a")]):
            self.date_naissance = datetime.datetime(
                self._naissance_a.year,
                self._naissance_m.month,
                self._naissance_j.day,
                0, 0, 0)
        if all([self._prelevement_date, self._prelevement_time]):
            self.date_prelevement = datetime.datetime(
                *(self._prelevement_date.timetuple()[:3] +
                    self._prelevement_time.timetuple()[3:5]))
        return self

    def to_string(self):
        if self.date_prelevement:
            self._prelevement_time = self.date_prelevement
            self._prelevement_date = self.date_prelevement
        if self.date_naissance:
            self._naissance_a = self.date_naissance
            self._naissance_m = self.date_naissance
            self._naissance_j = self.date_naissance
        values = [f.to_string(getattr(self, f.name)) for f in self.FIELDS]
        values.extend([r.to_string() for r in self.results])
        return self.SEP.join(values)


class GentianeUpload(GentianeBase):
    FIELDS = (
        StringSplitted("dossier"),
        Multiple("results", (
            StringSplitted("code_test"),
            StringSplitted("value"),
        ), separator=":"),
    )


class Gentiane(list):

    def add(self, *args, **kwargs):
        obj = GentianeResult(*args, **kwargs)
        self.append(obj)
        return obj

    def parse(self, data):

        for line in data.splitlines():
            if not line:
                continue
            if len(line.split(";")) == 18:
                self.append(GentianeResult().parse(line))
            else:
                self.append(GentianeUpload().parse(line))
        return self

    def to_string(self):
        return "\n".join([e.to_string() for e in self])


class Driver(Message):
    SPEC = {
        "code": "gentiane",
        "label": "Gentiane",
    }
    OPTIONS = [
        Message.BooleanOption(
            "service_prioritaire",
            label="Envoyer le service prioritairement au code médecin",
            default=False,
        ),
    ]

    def __init__(self, service_prioritaire=False, **kwargs):
        self.service_prioritaire = service_prioritaire
        super(Driver, self).__init__(**kwargs)

    def create(self):
        return Gentiane()

    def parse(self, data):
        return Gentiane().parse(data)

    def to_string(self, message):
        return message.to_string()

    def get_order_message(self, worklist, context):
        msg = self.create()
        for sample in worklist.samples:
            if self.service_prioritaire:
                if sample.service:
                    medecin = sample.service_name or sample.service
                else:
                    medecin = sample["physician"]
            elif sample["physician"]:
                medecin = sample["physician"]
            elif sample.service:
                medecin = sample.service
            else:
                medecin = ''
            adrs = sample["address"].split('~', 1)
            if len(adrs) > 1:
                adr2 = adrs[1]
            else:
                adr2 = ''
            message = msg.add(
                header="0000",
                identifiant=sample["patient_id"],
                nom_nomjf="%(last_name)s/%(maiden_name)s" % sample,
                prenom=sample["first_name"],
                date_naissance=sample["birthdate"],
                sexe=sample["sex"],
                date_prelevement=sample["collection_date"],
                numero_dossier=sample["sample_id"],
                addresse1=adrs[0],
                addresse2=adr2,
                code_postal=sample["postal_code"],
                ville=sample["city"],
                medecin=medecin,
            )
            message.nb_param = "%s" % (len(sample["results"]),)
            for result in sample["results"]:
                message.results.add(
                    code=result["code"],
                    value=result['value']
                )
        if len(msg):
            return msg
        else:
            return None

    def get_result_dict(self, message):
        results = {}
        for m in message:
            sample_id = m.dossier
            results.setdefault(
                sample_id,
                {"resultats": []},
            )
            for result in m.results:
                results[sample_id]["resultats"].append({
                    "test": result.code_test,
                    "resu": result.value
                })
        return results
